from sqlalchemy import Column, func, ForeignKey
from sqlalchemy.orm import relationship
from sqlalchemy.dialects.postgresql import UUID, JSON, TIMESTAMP, TEXT, INTEGER, VARCHAR, JSONB, SMALLINT, BOOLEAN, FLOAT
from uuid import uuid4
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.types import TypeDecorator
from sqlalchemy import DateTime as SdateTime
from sqlalchemy.schema import Sequence
from datetime import datetime, timezone
import pytz

#Mexico city time zone
mx_cty_tz = pytz.timezone('America/Mexico_City')

#Function to get timezone aware current time
now = lambda : datetime.utcnow().replace(tzinfo=timezone.utc).astimezone(tz=mx_cty_tz)

Base = declarative_base()

class DateTime(TypeDecorator):
    impl = SdateTime
    def process_bind_param(self, value, engine):
        if value is None:
            return value
        return value.replace(tzinfo=timezone.utc).astimezone(tz=mx_cty_tz)
    def process_result_value(self, value, engine):  
        return value

class AlembicVersion(Base):
    __tablename__ = "alembic_version"
    version_num = Column(VARCHAR(32), primary_key=True)

class Task(Base):
    __tablename__ = "tasks"
    uuid = Column(UUID(as_uuid=True), primary_key=True, default=uuid4)
    identifier = Column(VARCHAR(50), index=True)
    data_schema = Column(TEXT)
    requests = relationship('TaskRequest', back_populates="task")
    max_execution_time = Column(INTEGER)  # Minutes
    created_at = Column(DateTime, default=now, nullable=False)
    updated_at = Column(DateTime, onupdate=now)
    stage_list = Column(JSONB)
    wrap_secs = Column(SMALLINT)
    clonable = Column(BOOLEAN)


class TaskRequest(Base):
    __tablename__ = 'task_requests'
    uuid = Column(UUID(as_uuid=True), primary_key=True, default=uuid4)
    task_uuid = Column(UUID(as_uuid=True), ForeignKey('tasks.uuid'), nullable=False, index=True)
    task = relationship('Task', back_populates="requests")
    result = relationship('TaskResult', uselist=False, back_populates="task_request")
    statuses = relationship('TaskStatus', back_populates="task_request")
    data = Column(JSON)
    callback = Column(TEXT)
    created_at = Column(DateTime, default=now, nullable=False)
    updated_at = Column(DateTime, onupdate=now)
    uuid_task_source = Column(UUID(as_uuid=True), ForeignKey(uuid), index=True)
    options = Column(JSONB, nullable=True)


class TaskResult(Base):
    __tablename__ = "task_results"
    uuid = Column(UUID(as_uuid=True), primary_key=True, default=uuid4)
    task_request_uuid = Column(UUID(as_uuid=True), ForeignKey('task_requests.uuid'), nullable=False, unique=True)
    task_request = relationship('TaskRequest', back_populates="result")
    status = Column(VARCHAR(50), index=True)
    data = Column(JSON)
    created_at = Column(DateTime, default=now, nullable=False)
    updated_at = Column(DateTime)


class TaskStatus(Base):
    __tablename__ = "task_statuses"
    uuid = Column(UUID(as_uuid=True), primary_key=True, default=uuid4)
    task_request_uuid = Column(UUID(as_uuid=True), ForeignKey('task_requests.uuid'), nullable=False, index=True)
    task_request = relationship('TaskRequest', back_populates="statuses")
    status = Column(VARCHAR(50), index=True)
    data = Column(JSON)
    created_at = Column(DateTime, default=now, nullable=False)

class Names(Base):
    __tablename__ = "names"
    id_name_seq = Sequence('names_id_name_seq')
    id_name = Column(INTEGER, id_name_seq, primary_key=True,server_default=id_name_seq.next_value())
    name = Column(VARCHAR(50))
    name_type = Column(VARCHAR(10))

class AddressNames(Base):
    __tablename__ = "address_names"
    address_names_id_seq = Sequence('address_names_id_seq')
    id_addres_name = Column(INTEGER, address_names_id_seq, primary_key=True,
        server_default=address_names_id_seq.next_value())
    addres_name = Column(VARCHAR(50))

class TaskSegment(Base):
    __tablename__ = "task_segment"
    task_segment_id = Column(VARCHAR(50), primary_key=True, nullable=False, index=True)
    task_request_uuid = Column(UUID(as_uuid=True), ForeignKey(TaskRequest.uuid), nullable=False)
    data_result = Column(JSONB)
    status_code = Column(VARCHAR(5))
    status = Column(VARCHAR(20))
    total_segments = Column(SMALLINT)
    integrated = Column(BOOLEAN)
    options = Column(JSONB)
    score = Column(FLOAT, default=0, nullable=False)
    creation_time = Column(DateTime, default=now, nullable=False)
    updated_time = Column(DateTime, onupdate=now)